@@ -45,7 +45,7 @@ my($MINX, $MINY, $MAXX, $MAXY);
 $MINX = $MINY = 1;
 $MAXX = $MAXY = 32000;
 my($version, %options, %cellhash, @documenttext, @keywords, @fontdecls);
-$version="0.1.5";
+$version="0.1.7";
 
 ##################
 # Function Calls #
@@ -56,7 +56,8 @@ sub new {
     $type = $newtype;
     $type = "sxw" unless ($type);
 
-    if ($opt) {if ($opt eq "debug") {$options{debug} = 1;}}
+    if ($opt) {if ($opt =~ /debug/) {$options{debug} = 1;}}
+    if ($opt) {if ($opt =~ /zip/) {$options{zip} = 1;}}
 
     # Set default values
     $cellhash{x} = 1;
@@ -65,7 +66,9 @@ sub new {
     $cellhash{ymax} = 1;
 
     # Formatting
-    $options{nextstyle} = 1;
+    $options{"nextstyle"} = 1;
+    $options{"nextstyle table-column"} = 2;
+    $options{"nextstyle table-cell"} = 1;
     $options{nextlist} = 1;
     $options{justify} = "left";
     $options{bold} = "off";
@@ -101,6 +104,33 @@ sub new {
     bless \$package;
 }
 
+sub ooolib_sxc_cell_StyleName {
+    # Uses these numbers to keep track of which styles have been created
+    my(@prop);
+
+    # Default style
+    if ($options{bold} ne "on" &&
+	$options{italic} ne "on" &&
+	$options{textcolor} eq "000000") {return "ro1";}
+
+    if ($options{bold} eq "on") {push(@prop, "fo:font-weight=\"bold\"");}
+    if ($options{italic} eq "on") {push(@prop, "fo:font-style=\"italic\"");}
+    if ($options{textcolor} ne "000000") {push(@prop, "fo:color=\"#$options{textcolor}\"");}
+
+    my($propline) = join(" ", @prop);
+
+    if ($option{"cell-style $propline"}) {return $option{"cell-style $propline"};}
+    my($num) = $options{"nextstyle table-cell"};
+    $options{"nextstyle table-cell"}++;
+    my($style) = "ce$num";
+
+    push(@ooolib_sxc_cell_styles, "<style:style style:name=\"$style\" style:family=\"table-cell\" style:parent-style-name=\"Default\">");
+    push(@ooolib_sxc_cell_styles, "<style:properties $propline/>");
+    push(@ooolib_sxc_cell_styles, "</style:style>");
+
+    return $style;
+}
+
 sub oooStyleName {
     # Returns the name of the style to use
     my($self) = shift(@_);
@@ -362,6 +392,7 @@ sub oooData {
 	$cellhash{"$x $y type"} = "float";
 	$cellhash{"$x $y value"} = $text;
 	$cellhash{"$x $y format"} = "";
+	$cellhash{"$x $y style"} = &ooolib_sxc_cell_StyleName;
 
 	&oooCellUpdate;
     } elsif ($style eq "cell-text") {
@@ -372,6 +403,7 @@ sub oooData {
 	$cellhash{"$x $y type"} = "text";
 	$cellhash{"$x $y value"} = $text;
 	$cellhash{"$x $y format"} = "";
+	$cellhash{"$x $y style"} = &ooolib_sxc_cell_StyleName;
 
 	&oooCellUpdate;
     } elsif ($style eq "cell-formula") {
@@ -382,6 +414,8 @@ sub oooData {
 	$cellhash{"$x $y type"} = "formula";
 	$cellhash{"$x $y value"} = $text;
 	$cellhash{"$x $y format"} = "";
+	$cellhash{"$x $y style"} = &ooolib_sxc_cell_StyleName;
+
 	&oooCellUpdate;
     } elsif ($style eq "cell-skip") {
 	# Does not do anything to the cell, just skips it
@@ -426,6 +460,7 @@ sub oooCleanText {
     my($text) = @_;
     $text =~ s/\&/\&amp;/g;
     $text =~ s/'/\&apos;/g;
+    $text =~ s/\"/\&quot;/g; # Path from Vladimir Vuksan
     $text =~ s/</\&lt;/g;
     $text =~ s/>/\&gt;/g;
     $text =~ s/\t/<text:tab-stop\/>/g;
@@ -473,9 +508,19 @@ sub oooSet {
     } elsif ($name eq "cell-loc") {
         # Set the writing cell
 	my($x, $y) = @_;
+
+	if ($x =~ /[a-zA-Z]/) {
+	    ($x, $y) = &ooosxcCellLocConv($x);
+	}
+
 	$cellhash{x} = $x;
 	$cellhash{y} = $y;
 	&oooCellCheck;
+    } elsif ($name eq "column-width") {
+	# Set width of column
+	my($col) = shift(@_);
+	my($width) = shift(@_);
+	$cellhash{"column $col width"} = $width;
     } elsif ($name eq "cell-left") {
 	# Subtract 1 from x
 	$cellhash{x}--;
@@ -765,12 +810,28 @@ sub oooGenerate {
     $cdloc = "cd $builddir";
 
     if ($options{debug}) {print "Building $type file from XML\n";}
-    system("$cdloc; zip -r '$filename' mimetype &> /dev/null");
-    system("$cdloc; zip -r '$filename' content.xml &> /dev/null");
-    system("$cdloc; zip -r '$filename' styles.xml &> /dev/null");
-    system("$cdloc; zip -r '$filename' meta.xml &> /dev/null");
-    system("$cdloc; zip -r '$filename' settings.xml &> /dev/null");
-    system("$cdloc; zip -r '$filename' META-INF/manifest.xml &> /dev/null");
+
+    # Decide which method to use.  zip command or Perl Module.
+    if ($options{zip}) {
+	# You will be required to pass the option zip to use the zip command
+	system("$cdloc; zip -r '$filename' mimetype > /dev/null 2> /dev/null");
+	system("$cdloc; zip -r '$filename' content.xml > /dev/null 2> /dev/null");
+	system("$cdloc; zip -r '$filename' styles.xml > /dev/null 2> /dev/null");
+	system("$cdloc; zip -r '$filename' meta.xml > /dev/null 2> /dev/null");
+	system("$cdloc; zip -r '$filename' settings.xml > /dev/null 2> /dev/null");
+	system("$cdloc; zip -r '$filename' META-INF/manifest.xml > /dev/null 2> /dev/null");
+    } else {
+	# The Archive::Zip perl module is now the default
+	use Archive::Zip;
+	my $zip = Archive::Zip->new();
+	$zip->addFile("$builddir/mimetype", 'mimetype');
+	$zip->addFile("$builddir/content.xml", 'content.xml');
+	$zip->addFile("$builddir/styles.xml", 'styles.xml');
+	$zip->addFile("$builddir/meta.xml", 'meta.xml');
+	$zip->addFile("$builddir/settings.xml", 'settings.xml');
+	$zip->addFile("$builddir/META-INF/manifest.xml", 'META-INF/manifest.xml');
+	$zip->writeToFileNamed("$builddir/$filename");
+    }
 
     # Clean up
     if ($options{debug}) {
@@ -790,6 +851,60 @@ sub oooGenerate {
 }
 
 # SXC specific function calls
+sub ooosxcCellLocConv {
+    # Convert A1 to 1,1
+    my($id) = @_;
+    my($xstr, $ystr, @xarray);
+    my($x, $y, $ch);
+
+    $id =~ tr/[a-z]/[A-Z]/;
+    $x = 0;
+
+    if($id =~ /([A-Z]+)([0-9]+)/) {
+	$xstr = $1;
+	$ystr = $2;
+
+	@xarray = split(//, $xstr);
+	while (@xarray) {
+	    $ch = shift(@xarray);
+
+	    # I forgot how to do this in one line.
+	    if ($ch eq "A") {$x += 1;}
+	    if ($ch eq "B") {$x += 2;}
+	    if ($ch eq "C") {$x += 3;}
+	    if ($ch eq "D") {$x += 4;}
+	    if ($ch eq "E") {$x += 5;}
+	    if ($ch eq "F") {$x += 6;}
+	    if ($ch eq "G") {$x += 7;}
+	    if ($ch eq "H") {$x += 8;}
+	    if ($ch eq "I") {$x += 9;}
+	    if ($ch eq "J") {$x += 10;}
+	    if ($ch eq "K") {$x += 11;}
+	    if ($ch eq "L") {$x += 12;}
+	    if ($ch eq "M") {$x += 13;}
+	    if ($ch eq "N") {$x += 14;}
+	    if ($ch eq "O") {$x += 15;}
+	    if ($ch eq "P") {$x += 16;}
+	    if ($ch eq "Q") {$x += 17;}
+	    if ($ch eq "R") {$x += 18;}
+	    if ($ch eq "S") {$x += 19;}
+	    if ($ch eq "T") {$x += 20;}
+	    if ($ch eq "U") {$x += 21;}
+	    if ($ch eq "V") {$x += 22;}
+	    if ($ch eq "W") {$x += 23;}
+	    if ($ch eq "X") {$x += 24;}
+	    if ($ch eq "Y") {$x += 25;}
+	    if ($ch eq "Z") {$x += 26;}
+
+	    # Increment the log
+	    if (@xarray) {$x *= 26;}
+	}
+
+	$y = $ystr+0;
+    }
+    return ($x, $y);
+}
+
 sub ooosxcCellLocation {
     # Set the writing cell
     my($self) = shift(@_);
@@ -882,8 +997,6 @@ sub oooWriteContent {
 	    print DATA "</office:font-decls>";
 	}
 
-
-
 	# Styles line bold, center, etc.
 	if (@autostyles) {
 	    my($autostyle);
@@ -963,56 +1076,95 @@ sub oooWriteContent {
 	# Styles
 	print DATA "<office:automatic-styles>";
 	
-	# Resize on column width
+	my($x, $y);
+	# table-column styles
+
 	print DATA "<style:style style:name=\"co1\" style:family=\"table-column\">";
 	print DATA "<style:properties fo:break-before=\"auto\" style:column-width=\"2.267cm\"/>";
 	print DATA "</style:style>";
+
+	# Default values
+	my($columnwidth) = "2.267";
+
+	for($x=1;$x<=$cellhash{xmax};$x++) {
+	    my($width) = $cellhash{"column $x width"};
+	    unless($width) {$width = $columnwidth;}
+	    if ($width ne $columnwidth) {
+		# Create a new style
+		my($num) = $options{"nextstyle table-column"};
+		$options{"nextstyle table-column"}++;
+		my($width) = sprintf("%5.3f",$width);
+		print DATA "<style:style style:name=\"co$num\" style:family=\"table-column\">";
+		print DATA "<style:properties fo:break-before=\"auto\" style:column-width=\"${width}cm\"/>";
+		print DATA "</style:style>";
+		$cellhash{"column $x style"} = "co$num";
+	    } else {
+		$cellhash{"column $x style"} = "co1";
+	    }
+	}
 	
+	# table-row styles
 	print DATA "<style:style style:name=\"ro1\" style:family=\"table-row\">";
 	print DATA "<style:properties fo:break-before=\"auto\"/>";
 	print DATA "</style:style>";
 	
+	# table styles
 	print DATA "<style:style style:name=\"ta1\" style:family=\"table\" style:master-page-name=\"Default\">";
 	print DATA "<style:properties table:display=\"true\"/>";
 	print DATA "</style:style>";
-	
+
+	# table-cell styles
+	if (@ooolib_sxc_cell_styles) {
+	    foreach $line (@ooolib_sxc_cell_styles) {
+		print DATA $line;
+	    }
+	}
+
 	print DATA "</office:automatic-styles>";
 	
 	# Beginning of document content
 	print DATA "<office:body>";
 	
 	print DATA "<table:table table:name=\"Sheet1\" table:style-name=\"ta1\">";
-	print DATA "<table:table-column table:style-name=\"co1\" table:number-columns-repeated=\"3\" table:default-cell-style-name=\"Default\"/>";
-	
-	my($x, $y);
-	# One row at a time
+
+	# First tell the document what each of the column style codes are.
+	for($x=1;$x<=$cellhash{xmax};$x++) {
+	    # Each Column
+	    my($style) = $cellhash{"column $x style"};
+	    print DATA "<table:table-column table:style-name=\"$style\" table:number-columns-repeated=\"1\" table:default-cell-style-name=\"Default\"/>";
+	}
+
+	# Row by row print the cells
 	for($y=1;$y<=$cellhash{ymax};$y++) {
-	    # One cell at a time down the row
+	    # One column cell at a time down the row
 	    print DATA "<table:table-row table:style-name=\"ro1\">";
 	    for($x=1;$x<=$cellhash{xmax};$x++) {
-		$celltype = $cellhash{"$x $y type"};
-		$value = $cellhash{"$x $y value"};
+		# Load information about the cell
+		my($celltype) = $cellhash{"$x $y type"};
+		my($value) = $cellhash{"$x $y value"};
+		my($style) = $cellhash{"$x $y style"};
+		unless($style) {$style = "ro1";}
+		
 		if ($celltype eq "text") {
-		    print DATA "<table:table-cell>";
+		    print DATA "<table:table-cell table:style-name=\"$style\">";
 		    print DATA "<text:p>$value</text:p>";
 		    print DATA "</table:table-cell>";
 		} elsif ($celltype eq "float") {
-		    print DATA "<table:table-cell table:value-type=\"float\" table:value=\"$value\">";
+		    print DATA "<table:table-cell table:value-type=\"float\" table:value=\"$value\" table:style-name=\"$style\">";
 		    print DATA "<text:p>$value</text:p>";
 		    print DATA "</table:table-cell>";
 		} elsif ($celltype eq "formula") {
 		    # cell-formula code from Vladimir Vuksan
-                    print DATA "<table:table-cell table:value-type=\"float\" table:formula=\"$value\" table:value=\"\">";
-                    print DATA "<text:p></text:p>";
-                    print DATA "</table:table-cell>";
+		    print DATA "<table:table-cell table:value-type=\"float\" table:formula=\"$value\" table:value=\"\" table:style-name=\"$style\">";
+		    print DATA "<text:p></text:p>";
+		    print DATA "</table:table-cell>";
 		} else {
 		    print DATA "<table:table-cell/>";
 		}
 	    }
 	    print DATA "</table:table-row>";
 	}
-	
-	
+
 	# Closing document
 	print DATA "</table:table>";
 	print DATA "</office:body>";
@@ -1517,6 +1669,10 @@ sub oooWriteManifest {
     
     # End of file
     print DATA "</manifest:manifest>\n";
+
+    # Forgotten close pointed out by Tom Dierickx
+    # http://www.data-for-all.com/blog/index.php?p=51
+    close(DATA);
 }
 
 sub oooTimeStamp {
